@@ -9,9 +9,8 @@ from contextlib import redirect_stdout
 from datetime import datetime, timedelta
 from itertools import product
 from os import getenv
-from os.path import dirname, realpath
 from pathlib import Path
-from string import Template
+from subprocess import run
 
 import click
 import discord
@@ -24,80 +23,26 @@ import pandas as pd
 import pytz
 from dateutil.relativedelta import relativedelta
 from humanize import naturaltime
-from yaml import load
+from unidecode import unidecode
 
 from turbot._version import __version__
+from turbot.assets import Assets, s
 from turbot.data import Data
 from turnips.archipelago import Archipelago
 from turnips.plots import plot_models_range
 
-try:
-    from yaml import CLoader as Loader
-except ImportError:  # pragma: no cover
-    from yaml import Loader
-
-
 matplotlib.use("Agg")
 
-PACKAGE_ROOT = Path(dirname(realpath(__file__)))
 RUNTIME_ROOT = Path(".")
+SCRIPTS_DIR = RUNTIME_ROOT / "scripts"
 DEFAULT_DB_DIR = RUNTIME_ROOT / "db"
-
-# application configuration files
 DEFAULT_CONFIG_TOKEN = RUNTIME_ROOT / "token.txt"
 DEFAULT_CONFIG_CHANNELS = RUNTIME_ROOT / "channels.txt"
-
-# static application asset data
-DATA_DIR = PACKAGE_ROOT / "data"
-STRINGS_DATA_FILE = DATA_DIR / "strings.yaml"
-FOSSILS_DATA_FILE = DATA_DIR / "fossils.txt"
-FISH_DATA_FILE = DATA_DIR / "fish.csv"
-BUGS_DATA_FILE = DATA_DIR / "bugs.csv"
-ART_DATA_FILE = DATA_DIR / "art.csv"
-
-# temporary application files
 TMP_DIR = RUNTIME_ROOT / "tmp"
 GRAPHCMD_FILE = TMP_DIR / "graphcmd.png"
 LASTWEEKCMD_FILE = TMP_DIR / "lastweek.png"
-
-# loaded application data
-STRINGS = None
-FISH = None
-BUGS = None
-ART = None
-FOSSILS_SET = None
-FISH_SET = None
-BUGS_SET = None
-ART_SET = None
-COLLECTABLE_SET = None
-
-
-def load_application_data():
-    """Defer loading these assests until we actually need them."""
-    global STRINGS
-    global FISH
-    global BUGS
-    global ART
-    global FOSSILS_SET
-    global FISH_SET
-    global BUGS_SET
-    global ART_SET
-    global COLLECTABLE_SET
-
-    with open(STRINGS_DATA_FILE) as f:
-        STRINGS = load(f, Loader=Loader)
-
-    FISH = pd.read_csv(FISH_DATA_FILE)
-    BUGS = pd.read_csv(BUGS_DATA_FILE)
-    ART = pd.read_csv(ART_DATA_FILE)
-
-    with open(FOSSILS_DATA_FILE) as f:
-        FOSSILS_SET = frozenset([line.strip().lower() for line in f.readlines()])
-    FISH_SET = frozenset(FISH.drop_duplicates(subset="name").name.tolist())
-    BUGS_SET = frozenset(BUGS.drop_duplicates(subset="name").name.tolist())
-    ART_SET = frozenset(ART.drop_duplicates(subset="name").name.tolist())
-    COLLECTABLE_SET = FOSSILS_SET | FISH_SET | BUGS_SET | ART_SET
-
+MIGRATIONS_FILE = DEFAULT_DB_DIR / "migrations.txt"
+MIGRATIONS_DIR = SCRIPTS_DIR / "migrations"
 
 EMBED_LIMIT = 5  # more embeds in a row than this causes issues
 
@@ -161,13 +106,6 @@ class Validate:
         return value
 
 
-def s(key, **kwargs):
-    """Returns a string from data/strings.yaml with subsitutions."""
-    data = STRINGS.get(key, "")
-    assert data, f"error: missing strings key: {key}"
-    return Template(data).substitute(kwargs)
-
-
 def h(dt):
     """Convertes a datetime to something readable by a human."""
     if hasattr(dt, "tz_convert"):  # pandas-datetime-like objects
@@ -183,6 +121,10 @@ def day_and_time(dt):
     return f"{day.title()} {am_pm}"
 
 
+def sanitize_collectable(item):
+    return unidecode(item.strip().lower())
+
+
 def humanize_months(row):
     """Generator that humanizes months from row data where each month is a column."""
     ABBR = {
@@ -315,7 +257,8 @@ class Turbot(discord.Client):
             if hasattr(member[1], "is_command") and member[1].is_command
         ]
 
-        load_application_data()
+        # load static application data
+        self.assets = Assets()
 
     def run(self):  # pragma: no cover
         super().run(self.token)
@@ -578,16 +521,16 @@ class Turbot(discord.Client):
                     breakpoint = index
                     break
 
-            # A trailing blank quoted line just shows the > symbol, so if possible
-            # we should try bumping that into the remainder if it is present
-            lastnlindex = remaining.rfind("\n", 1800, breakpoint - 1)
-            finalline = remaining[lastnlindex:breakpoint]
-            if finalline.strip() == ">":
-                # the breakpoint should actually be bumped back a bit
-                breakpoint = lastnlindex
-
-            yield remaining[0 : breakpoint + 1]
+            message = remaining[0 : breakpoint + 1]
+            yield message
             remaining = remaining[breakpoint + 1 :]
+            last_line_end = message.rfind("\n")
+            if last_line_end != -1 and len(message) > last_line_end + 1:
+                last_line_start = last_line_end + 1
+            else:
+                last_line_start = 0
+            if message[last_line_start] == ">":
+                remaining = f"> {remaining}"
 
         yield remaining
 
@@ -689,10 +632,10 @@ class Turbot(discord.Client):
     # Would trigger the bot to send three messages to the channel with no attachment.
     #
     # The docstring used for the command method will be automatically used as the help
-    # message for the command. To document commands with parameters use a | to delimit
+    # message for the command. To document commands with parameters use a @ to delimit
     # the help message from the parameter documentation. For example:
     #
-    #     """This is the help message for your command. | [and] [these] [are] [params]"""
+    #     """This is the help message for your command. @ [and] [these] [are] [params]"""
     #
     # A [parameter] is optional whereas a <parameter> is required.
 
@@ -701,21 +644,21 @@ class Turbot(discord.Client):
         """
         Shows this help screen.
         """
-        usage = "__**Turbot Help!**__"
+        usage = "__**Turbot Help!**__\n"
         for command in self.commands:
             method = getattr(self, command)
-            doc = method.__doc__.split("|")
+            doc = method.__doc__.split("@")
             use, params = doc[0], ", ".join([param.strip() for param in doc[1:]])
             use = inspect.cleandoc(use)
             use = use.replace("\n", " ")
 
-            title = f"!{command}"
+            title = f"**!{command}**"
             if params:
-                title = f"{title} {params}"
-            usage += f"\n> **{title}**"
-            usage += f"\n>    {use}"
-            usage += "\n> "
-        usage += "\n> turbot created by TheAstropath"
+                title = f"{title} _{params}_"
+            usage += f"\n{title}"
+            usage += f"\n>  {use}"
+            usage += "\n"
+        usage += "\n_Turbot created by TheAstropath_"
         return usage, None
 
     class _PriceTimeError(Exception):
@@ -748,7 +691,7 @@ class Turbot(discord.Client):
     def sell(self, channel, author, params):
         """
         Log the price that you can sell turnips for on your island.
-        | <price> [day time]
+        @ <price> [day time-of-day]
         """
         if not params:
             return s("sell_no_params"), None
@@ -786,7 +729,7 @@ class Turbot(discord.Client):
     def buy(self, channel, author, params):
         """
         Log the price that you can buy turnips from Daisy Mae on your island.
-        | <price> [day time]
+        @ <price> [day time]
         """
         if not params:
             return s("buy_no_params"), None
@@ -850,7 +793,7 @@ class Turbot(discord.Client):
     def history(self, channel, author, params):
         """
         Show the historical turnip prices for a user. If no user is specified, it will
-        display your own prices. | [user]
+        display your own prices. @ [user]
         """
         target = author.id if not params else params[0]
         target_name = discord_user_name(channel, target)
@@ -901,7 +844,7 @@ class Turbot(discord.Client):
         """
         Finds the best, most recent, buy or sell price currently available.
         The default is to look for the best sell.
-        | [buy|sell]
+        @ [buy|sell]
         """
         kind = params[0].lower() if params else "sell"
         if kind not in ["sell", "buy"]:
@@ -923,18 +866,13 @@ class Turbot(discord.Client):
     def collect(self, channel, author, params):
         """
         Mark collectables as donated to your museum. The names must match the in-game item
-        name exactly. | <comma, separated, list, of, things>
+        name exactly. @ <comma, separated, list, of, things>
         """
         if not params:
             return s("collect_no_params"), None
 
-        items = set(item.strip().lower() for item in " ".join(params).split(","))
-
-        valid_fossils = items.intersection(FOSSILS_SET)
-        valid_bugs = items.intersection(BUGS_SET)
-        valid_fish = items.intersection(FISH_SET)
-        valid_art = items.intersection(ART_SET)
-        invalid = items.difference(COLLECTABLE_SET)
+        items = set(sanitize_collectable(item) for item in " ".join(params).split(","))
+        valid, invalid = self.assets.validate(items)
 
         lines = []
 
@@ -944,23 +882,21 @@ class Turbot(discord.Client):
             store = getattr(self.data, kind)
             yours = store[store.author == author.id]
             dupes = yours.loc[yours.name.isin(valid_items)].name.values.tolist()
-            new_names = list(set(valid_items) - set(dupes))
-            new_data = [[author.id, name] for name in new_names]
+            new_items = list(set(valid_items) - set(dupes))
+            new_data = [[author.id, name] for name in new_items]
             new_fossils = pd.DataFrame(columns=store.columns, data=new_data)
             store = store.append(new_fossils, ignore_index=True)
             yours = store[store.author == author.id]  # re-fetch for congrats
             self.data.commit(store)
-            if new_names:
-                lines.append(s(f"collect_{kind}_new", items=", ".join(sorted(new_names))))
+            if new_items:
+                lines.append(s(f"collect_{kind}_new", items=", ".join(sorted(new_items))))
             if dupes:
                 lines.append(s(f"collect_{kind}_dupe", items=", ".join(sorted(dupes))))
             if len(fullset) == len(yours.index):
                 lines.append(s(f"congrats_all_{kind}"))
 
-        add_lines("fossils", valid_fossils, FOSSILS_SET)
-        add_lines("bugs", valid_bugs, BUGS_SET)
-        add_lines("fish", valid_fish, FISH_SET)
-        add_lines("art", valid_art, ART_SET)
+        for kind in self.assets.collectables:
+            add_lines(kind, valid[kind], self.assets[kind].all)
 
         if invalid:
             lines.append(s("invalid_collectable", items=", ".join(sorted(invalid))))
@@ -971,18 +907,13 @@ class Turbot(discord.Client):
     def uncollect(self, channel, author, params):
         """
         Unmark collectables as donated to your museum. The names must match the in-game
-        item name exactly. | <comma, separated, list, of, things>
+        item name exactly. @ <comma, separated, list, of, things>
         """
         if not params:
             return s("uncollect_no_params"), None
 
-        items = set(item.strip().lower() for item in " ".join(params).split(","))
-
-        valid_fossils = items.intersection(FOSSILS_SET)
-        valid_bugs = items.intersection(BUGS_SET)
-        valid_fish = items.intersection(FISH_SET)
-        valid_art = items.intersection(ART_SET)
-        invalid = items.difference(COLLECTABLE_SET)
+        items = set(sanitize_collectable(item) for item in " ".join(params).split(","))
+        valid, invalid = self.assets.validate(items)
 
         lines = []
 
@@ -1001,10 +932,8 @@ class Turbot(discord.Client):
                 items_str = ", ".join(sorted(didnt_have))
                 lines.append(s(f"uncollect_{kind}_already", items=items_str))
 
-        add_lines("fossils", valid_fossils)
-        add_lines("bugs", valid_bugs)
-        add_lines("fish", valid_fish)
-        add_lines("art", valid_art)
+        for kind in self.assets.collectables:
+            add_lines(kind, valid[kind])
 
         if invalid:
             lines.append(s("invalid_collectable", items=", ".join(sorted(invalid))))
@@ -1016,18 +945,13 @@ class Turbot(discord.Client):
         """
         Searches all users to see who needs the given collectables. The names must match
         the in-game item name, and more than one can be provided if separated by commas.
-        | <list of collectables>
+        @ <comma, separated, list, of, collectables>
         """
         if not params:
             return s("search_no_params"), None
 
-        items = set(item.strip().lower() for item in " ".join(params).split(","))
-
-        valid_fossils = items.intersection(FOSSILS_SET)
-        valid_bugs = items.intersection(BUGS_SET)
-        valid_fish = items.intersection(FISH_SET)
-        valid_art = items.intersection(ART_SET)
-        invalid = items.difference(COLLECTABLE_SET)
+        items = set(sanitize_collectable(item) for item in " ".join(params).split(","))
+        valid, invalid = self.assets.validate(items)
 
         def get_results(kind, valid_items):
             store = getattr(self.data, kind)
@@ -1041,39 +965,29 @@ class Turbot(discord.Client):
                     results[name].append(collected_item)
             return results
 
-        fossils_results = get_results("fossils", valid_fossils)
-        bugs_results = get_results("bugs", valid_bugs)
-        fish_results = get_results("fish", valid_fish)
-        art_results = get_results("art", valid_art)
+        results = {}
+        for kind in self.assets.collectables:
+            results[kind] = get_results(kind, valid[kind])
 
-        if not fossils_results and not art_results and not invalid:
+        if not any(result for result in results.values()) and not invalid:
             return s("search_all_not_needed"), None
 
-        searched = valid_fossils | valid_bugs | valid_fish | valid_art
+        searched = set()
+        for valid_items in valid.values():
+            searched.update(valid_items)
+
         needed = set()
-        for items in fossils_results.values():
-            needed.update(items)
-        for items in fish_results.values():
-            needed.update(items)
-        for items in bugs_results.values():
-            needed.update(items)
-        for items in art_results.values():
-            needed.update(items)
+        for result in results.values():
+            for items in result.values():
+                needed.update(items)
+
         not_needed = searched - needed
 
         lines = []
-        for name, items in fossils_results.items():
-            items_str = ", ".join(sorted(items))
-            lines.append(s("search_fossil_row", name=name, items=items_str))
-        for name, items in fish_results.items():
-            items_str = ", ".join(sorted(items))
-            lines.append(s("search_fish_row", name=name, items=items_str))
-        for name, items in bugs_results.items():
-            items_str = ", ".join(sorted(items))
-            lines.append(s("search_bugs_row", name=name, items=items_str))
-        for name, items in art_results.items():
-            items_str = ", ".join(sorted(items))
-            lines.append(s("search_art_row", name=name, items=items_str))
+        for kind, result in results.items():
+            for name, items in result.items():
+                items_str = ", ".join(sorted(items))
+                lines.append(s(f"search_{kind}_row", name=name, items=items_str))
         if not_needed:
             items_str = ", ".join(sorted(not_needed))
             lines.append(s("search_not_needed", items=items_str))
@@ -1085,7 +999,7 @@ class Turbot(discord.Client):
     def uncollected(self, channel, author, params):
         """
         Lists all collectables that you still need to donate. If a user is provided, it
-        gives the same information for that user instead. | [user]
+        gives the same information for that user instead. @ [user]
         """
         target = author.id if not params else params[0]
         target_name = discord_user_name(channel, target)
@@ -1110,10 +1024,8 @@ class Turbot(discord.Client):
             else:
                 lines.append(s(f"congrats_all_{kind}"))
 
-        add_lines("fossils", FOSSILS_SET)
-        add_lines("bugs", BUGS_SET)
-        add_lines("fish", FISH_SET)
-        add_lines("art", ART_SET)
+        for kind in self.assets.collectables:
+            add_lines(kind, self.assets[kind].all)
 
         return "\n".join(lines), None
 
@@ -1122,24 +1034,16 @@ class Turbot(discord.Client):
         """
         Lists all the needed items for all the channel members. As the only parameter
         give the name of the kind of collectable to return.
-        | <fossils|bugs|fish|art>
+        @ <fossils|bugs|fish|art|songs>
         """
         if not params:
             return s("needed_no_param"), None
 
         kind = params[0].lower()
-        if kind not in ["fossils", "bugs", "fish", "art"]:
+        if kind not in ["fossils", "bugs", "fish", "art", "songs"]:
             return s("needed_invalid_param"), None
 
-        if kind == "fossils":
-            fullset = FOSSILS_SET
-        if kind == "bugs":
-            fullset = BUGS_SET
-        if kind == "fish":
-            fullset = FISH_SET
-        if kind == "art":
-            fullset = ART_SET
-
+        fullset = self.assets[kind].all
         store = getattr(self.data, kind)
         authors = [member.id for member in channel.members if member.id != self.user.id]
         total = pd.DataFrame(list(product(authors, fullset)), columns=store.columns)
@@ -1158,10 +1062,10 @@ class Turbot(discord.Client):
                 items_str = s("needed_lots", limit=limit, kind=kind)
             else:
                 items_str = ", ".join(items_list)
-            lines.append(s(f"needed", name=name, items=items_str))
+            lines.append(s("needed", name=name, items=items_str))
 
         if not lines:
-            return s(f"needed_none", kind=kind), None
+            return s("needed_none", kind=kind), None
 
         return "\n".join(sorted(lines)), None
 
@@ -1169,7 +1073,7 @@ class Turbot(discord.Client):
     def collected(self, channel, author, params):
         """
         Lists all collectables that you have already donated. If a user is provided, it
-        gives the same information for that user instead. | [user]
+        gives the same information for that user instead. @ [user]
         """
         target = author.id if not params else params[0]
         target_name = discord_user_name(channel, target)
@@ -1183,17 +1087,12 @@ class Turbot(discord.Client):
             return set(your_items.name.unique())
 
         collected_items = {
-            "fossils": get_collection("fossils"),
-            "bugs": get_collection("bugs"),
-            "fish": get_collection("fish"),
-            "art": get_collection("art"),
+            kind: get_collection(kind) for kind in self.assets.collectables
         }
 
         all_items = {
-            "fossils": len(collected_items["fossils"]) == len(FOSSILS_SET),
-            "fish": len(collected_items["fish"]) == len(FISH_SET),
-            "bugs": len(collected_items["bugs"]) == len(BUGS_SET),
-            "art": len(collected_items["art"]) == len(ART_SET),
+            kind: len(collected_items[kind]) == len(self.assets[kind].all)
+            for kind in self.assets.collectables
         }
 
         lines = []
@@ -1217,7 +1116,7 @@ class Turbot(discord.Client):
     @command
     def predict(self, channel, author, params):
         """
-        Get a link to a prediction calculator for a price history. | [user]
+        Get a link to a prediction calculator for a price history. @ [user]
         """
         target = author.id if not params else params[0]
         target_name = discord_user_name(channel, target)
@@ -1239,7 +1138,7 @@ class Turbot(discord.Client):
     @command
     def pref(self, channel, author, params):
         """
-        Set one of your user preferences. | <preference> <value>
+        Set one of your user preferences. @ <preference> <value>
         """
         if not params:
             return s("pref_no_params", prefs=", ".join(USER_PREFRENCES)), None
@@ -1262,22 +1161,23 @@ class Turbot(discord.Client):
     def count(self, channel, author, params):
         """
         Provides a count of the number of pieces of collectables for the comma-separated
-        list of users. | <list of users>
+        list of users. @ [comma, separated, list, of, users]
         """
         if not params:
-            return s("count_no_params"), None
-
-        users = set(item.strip().lower() for item in " ".join(params).split(","))
-
-        valid = []
-        invalid = []
-        for user in users:
-            user_name = discord_user_name(channel, user)
-            user_id = discord_user_id(channel, user_name)
-            if user_name and user_id:
-                valid.append((user_name, user_id))
-            else:
-                invalid.append(user)
+            user = discord_user_from_id(channel, author.id)
+            valid = [(user.name, user.id)]
+            invalid = []
+        else:
+            users = set(sanitize_collectable(p) for p in " ".join(params).split(","))
+            valid = []
+            invalid = []
+            for user in users:
+                user_name = discord_user_name(channel, user)
+                user_id = discord_user_id(channel, user_name)
+                if user_name and user_id:
+                    valid.append((user_name, user_id))
+                else:
+                    invalid.append(user)
 
         lines = []
 
@@ -1292,10 +1192,8 @@ class Turbot(discord.Client):
                 lines.append(s(f"count_{kind}_valid", name=user_name, count=count))
 
         if valid:
-            add_valid_lines("fossils", FOSSILS_SET)
-            add_valid_lines("bugs", BUGS_SET)
-            add_valid_lines("fish", FISH_SET)
-            add_valid_lines("art", ART_SET)
+            for kind in self.assets.collectables:
+                add_valid_lines(kind, self.assets[kind].all)
 
         if invalid:
             lines.append(s("count_invalid_header"))
@@ -1307,19 +1205,22 @@ class Turbot(discord.Client):
     @command
     def art(self, channel, author, params):
         """
-        Get info about pieces of art that are available | [List of art pieces]
+        Get info about pieces of art that are available
+        @ [comma, separated, list, of, art, pieces]
         """
         response = ""
         if params:
-            items = set(item.strip().lower() for item in " ".join(params).split(","))
-            validset = items.intersection(ART_SET)
-            invalidset = items - validset
-            valid = sorted(list(validset))
+            items = set(
+                sanitize_collectable(item) for item in " ".join(params).split(",")
+            )
+            validset, invalidset = self.assets.validate(items, kinds=["art"])
+            art_data = self.assets["art"].data
+            valid = sorted(list(validset["art"]))
             invalid = sorted(list(invalidset))
             lines = []
             response = s("art_header") + "\n"
             for art in valid:
-                piece = ART[ART.name == art].iloc[0]
+                piece = art_data[art_data.name == art].iloc[0]
                 if piece["has_fake"]:
                     lines.append(
                         s(
@@ -1345,7 +1246,8 @@ class Turbot(discord.Client):
                 response += "\n" + (s("art_invalid", items=", ".join(invalid)))
 
         else:
-            response = s("allart", list=", ".join(sorted(ART_SET)))
+            items = self.assets["art"].all
+            response = s("allart", list=", ".join(sorted(items)))
 
         return response, None
 
@@ -1398,7 +1300,7 @@ class Turbot(discord.Client):
                         if lhs_hour <= now_hour <= rhs_hour:
                             yield row["name"]
 
-    def _creatures(self, *_, author, params, kind, source, force_text=False):
+    def _creatures(self, *, author, params, kind, source, force_text=False):
         """The fish and bugs commands are so similar; I factored them out to a helper."""
         hemisphere = self.get_user_prefs(author.id).get("hemisphere", None)
         if not hemisphere:
@@ -1517,18 +1419,20 @@ class Turbot(discord.Client):
     def fish(self, channel, author, params):
         """
         Tells you what fish are available now in your hemisphere.
-        | [name|leaving|arriving]
+        @ [name|leaving|arriving]
         """
-        found = self._creatures(author=author, params=params, kind="fish", source=FISH)
+        source = self.assets["fish"].data
+        found = self._creatures(author=author, params=params, kind="fish", source=source)
         return found, None
 
     @command
     def bugs(self, channel, author, params):
         """
         Tells you what bugs are available now in your hemisphere.
-        | [name|leaving|arriving]
+        @ [name|leaving|arriving]
         """
-        found = self._creatures(author=author, params=params, kind="bugs", source=BUGS)
+        source = self.assets["bugs"].data
+        found = self._creatures(author=author, params=params, kind="bugs", source=source)
         return found, None
 
     @command
@@ -1536,11 +1440,21 @@ class Turbot(discord.Client):
         """
         Tells you what new things available in your hemisphere right now.
         """
+        bugs_data = self.assets["bugs"].data
         bugs = self._creatures(
-            author=author, params=["arriving"], kind="bugs", source=BUGS, force_text=True
+            author=author,
+            params=["arriving"],
+            kind="bugs",
+            source=bugs_data,
+            force_text=True,
         )
+        fish_data = self.assets["fish"].data
         fish = self._creatures(
-            author=author, params=["arriving"], kind="fish", source=FISH, force_text=True
+            author=author,
+            params=["arriving"],
+            kind="fish",
+            source=fish_data,
+            force_text=True,
         )
         return [*bugs, *fish], None
 
@@ -1588,7 +1502,7 @@ class Turbot(discord.Client):
     @command
     def info(self, channel, author, params):
         """
-        Gives you information on a user. | [user]
+        Gives you information on a user. @ [user]
         """
         if len(params) < 1:
             return s("info_no_params"), None
@@ -1660,7 +1574,11 @@ def get_token(token_file):  # pragma: no cover
 
 
 def get_channels(channels_file):  # pragma: no cover
-    """Returns the authorized channels your channels config file."""
+    """Returns the authorized channels the environment or your channels config file."""
+    channels = getenv("TURBOT_CHANNELS", None)
+    if channels:
+        return channels.split(";")
+
     try:
         with open(channels_file, "r") as channels_file:
             return [line.strip() for line in channels_file.readlines()]
@@ -1668,6 +1586,31 @@ def get_channels(channels_file):  # pragma: no cover
         return []
 
 
+def apply_migrations():  # pragma: no cover
+    """Applies any migration scripts that haven't already been applied."""
+
+    def migration_key(script):
+        return int(script.name[0:3])
+
+    applied = set()
+    if MIGRATIONS_FILE.exists():
+        with open(MIGRATIONS_FILE) as f:
+            applied.update(line.strip() for line in f.readlines())
+    with open(MIGRATIONS_FILE, "a") as f:
+        for migration in sorted(MIGRATIONS_DIR.glob("*.py"), key=migration_key):
+            if migration.name not in applied:
+                print(f"applying migration {migration.name}...", end=" ")
+                cmd = ["python3", migration]
+                proc = run(cmd, capture_output=True)
+                if proc.returncode != 0:
+                    print()
+                    print(proc.stdout.decode("utf-8"))
+                    print(proc.stderr.decode("utf-8"), file=sys.stderr)
+                    sys.exit(1)
+                print("done")
+                f.write(f"{migration.name}\n")
+
+
 @click.command()
 @click.option(
     "-l",
@@ -1687,13 +1630,20 @@ def get_channels(channels_file):  # pragma: no cover
     "-c",
     "--channel",
     multiple=True,
-    help="authorize a channel; use this multiple times to authorize multiple channels",
+    help=(
+        "authorize a channel; use this multiple times to authorize multiple channels; "
+        "you can also set the list of channels via the TURBOT_CHANNELS environment "
+        "variable separated by ; (semicolon)."
+    ),
 )
 @click.option(
     "-a",
     "--auth-channels-file",
     default=DEFAULT_CONFIG_CHANNELS,
-    help="read authorized channel names from this file",
+    help=(
+        "read channel names from this file; you can also set the list of channels via "
+        "the TURBOT_CHANNELS environment variable separated by ; (semicolon)."
+    ),
 )
 @click.option(
     "--db-dir", default=DEFAULT_DB_DIR, help="use this directory for user db files"
@@ -1713,22 +1663,24 @@ def main(
         print("error: you must provide at least one authorized channel", file=sys.stderr)
         sys.exit(1)
 
-    if dev:
-        reloader = hupper.start_reloader("turbot.main")
-        reloader.watch_files(
-            [ART_DATA_FILE, BUGS_DATA_FILE, FISH_DATA_FILE, STRINGS_DATA_FILE]
-        )
-
     # ensure transient application directories exist
     db_dir.mkdir(exist_ok=True)
     TMP_DIR.mkdir(exist_ok=True)
 
-    Turbot(
+    client = Turbot(
         token=get_token(bot_token_file),
         channels=auth_channels,
         db_dir=db_dir,
         log_level=getattr(logging, "DEBUG" if verbose else log_level),
-    ).run()
+    )
+
+    apply_migrations()
+
+    if dev:
+        reloader = hupper.start_reloader("turbot.main")
+        reloader.watch_files(client.assets.files())
+
+    client.run()
 
 
 if __name__ == "__main__":
